@@ -40,6 +40,8 @@ AAptInterface::AAptInterface(QObject* parent = 0) : QObject(parent)
 	iNeedRepoRefresh = false;
 	iNeedListRefresh = true;
 	iNeedDateRefresh = true;
+	iNeedDpkgRefresh = true;
+	iSkipRefreshListAndDates = false;
 
 	iProcAptGetUpdate = new QProcess(this);
 	iProcAptGetSimulate = new QProcess(this);
@@ -69,8 +71,22 @@ AAptInterface::AAptInterface(QObject* parent = 0) : QObject(parent)
 	iProcessPackagesOrig.clear();
 	iProcessPackageVersions.clear();
 
-	QDir logdir(KLogFileDir);
-	logdir.mkpath(KLogFileDir);
+	QDir logdir(KFapmanDir);
+	logdir.mkpath(KFapmanDir);
+
+	// create custom apt cache dirs if they don't exist
+	QDir d1(KAptListDir + "/partial");
+	d1.mkpath(KAptListDir + "/partial");
+
+	// clear log on startup
+	QFile logfile(KLogFile);
+	logfile.remove();
+
+	// clear files left from old versions
+	QFile f1("/root/.fapman/dates.cache");
+	f1.remove();
+	QFile f2("/root/.fapman/lastupdate");
+	f2.remove();
 
 	readRepositoryInfo();
 }
@@ -92,6 +108,26 @@ void AAptInterface::addQueuedOperation(interfaceMode mode_)
 	iOperationsQueue.append( mode_ );
 }
 
+void AAptInterface::setNeedRefresh(int repos, int lists, int dpkg, int dates)
+{
+	if( repos==0 || repos==1 )
+		iNeedRepoRefresh = (bool)repos;
+	if( lists==0 || lists==1 )
+		iNeedListRefresh = (bool)lists;
+	if( dpkg==0 || dpkg==1 )
+		iNeedDpkgRefresh = (bool)dpkg;
+	if( dates==0 || dates==1 )
+		iNeedDateRefresh = (bool)dates;
+}
+
+bool AAptInterface::needListOrDateRefresh()
+{
+	if( iNeedListRefresh || iNeedDpkgRefresh || iNeedDateRefresh )
+		return true;
+
+	return false;
+}
+
 bool AAptInterface::run(dimmer* uiDimmer)
 {
 	if( iMode != ModeNone ) {
@@ -164,6 +200,7 @@ void AAptInterface::cleanAfterRunAll()
 {
 	cleanAfterRunEach();
 	iUiDimmer = 0;
+	iSkipRefreshListAndDates = false;
 }
 
 void AAptInterface::cleanAfterError()
@@ -239,6 +276,8 @@ bool AAptInterface::startAptGetUpdate()
 	if( !this->writeRepositories() )
 		return false;
 	iNeedListRefresh = true;
+	iNeedDpkgRefresh = true;
+	iNeedDateRefresh = true;
 
 	if( iUiDimmer ) {
 		iUiDimmer->setProgress(0);
@@ -269,8 +308,10 @@ bool AAptInterface::startAptGetUpdate()
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "-q" << "update";
-
+	runParameters << "-q" << "update"
+			<< "-o" << "Dir::Etc::sourcelist=" + KAptSourceList
+			<< "-o" << "Dir::State::lists=" + KAptListDir
+			<< "-o"	<< "Dir::Etc::sourceparts=\"\"";
 	logToFile( runBinary + " " + runParameters.join(" ") );
 	iProcAptGetUpdate->start(runBinary,runParameters);
 
@@ -290,8 +331,11 @@ bool AAptInterface::startAptGetSimulate()
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
 	runParameters << "-qsy" << "--allow-unauthenticated";
-	if( iSettings->qsettings()->value("enable_autoremove", false).toBool() )
-		runParameters << "--auto-remove";
+	if( iSettings->qsettings()->value("enable_autoremove", true).toBool() )
+		runParameters << "--auto-remove"
+				<< "-o" << "Dir::Etc::sourcelist=" + KAptSourceList
+				<< "-o" << "Dir::State::lists=" + KAptListDir
+				<< "-o"	<< "Dir::Etc::sourceparts=\"\"";
 	runParameters << "install";
 	runParameters << iProcessPackages;
 
@@ -307,6 +351,7 @@ bool AAptInterface::startAptGetInstall()
 		return false;
 
 	iNeedListRefresh = true;
+	iNeedDpkgRefresh = true;
 
 	iProcAptGetInstallOutput.clear();
 
@@ -314,8 +359,12 @@ bool AAptInterface::startAptGetInstall()
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "-qy" << "--allow-unauthenticated" << "-o" << "DPkg::options::=--force-confnew";
-	if( iSettings->qsettings()->value("enable_autoremove", false).toBool() )
+	runParameters << "-qy" << "--allow-unauthenticated"
+			<< "-o" << "DPkg::options::=--force-confnew"
+			<< "-o" << "Dir::Etc::sourcelist=" + KAptSourceList
+			<< "-o" << "Dir::State::lists=" + KAptListDir
+			<< "-o"	<< "Dir::Etc::sourceparts=\"\"";
+	if( iSettings->qsettings()->value("enable_autoremove", true).toBool() )
 		runParameters << "--auto-remove";
 	runParameters << "install";
 	runParameters << iProcessPackagesOrig;
@@ -362,7 +411,11 @@ bool AAptInterface::startAptGetClean()
 {
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "clean";
+	runParameters
+			<< "-o" << "Dir::Etc::sourcelist=" + KAptSourceList
+			<< "-o" << "Dir::State::lists=" + KAptListDir
+			<< "-o"	<< "Dir::Etc::sourceparts=\"\""
+			<< "clean";
 
 	if( iUiDimmer ) {
 		iUiDimmer->updateText("Cleaning package cache");
@@ -514,7 +567,7 @@ void AAptInterface::finishedAptGetUpdate(int exitCode, QProcess::ExitStatus exit
 	if( success ) {
 		iNeedRepoRefresh = false;
 
-		QFile lastupdate("/root/.fapman/lastupdate");   // create an empty file and/or update the modification time
+		QFile lastupdate(KLastUpdateFile);   // create an empty file and/or update the modification time
 		if( lastupdate.open(QIODevice::WriteOnly) )
 			lastupdate.close();
 
@@ -792,7 +845,7 @@ void AAptInterface::progressCheckTimerCallback()
 	if( iAptGetCurrentDownloadFileName=="" )
 		return;
 
-	QFile pkgfile("/var/cache/apt/archives/partial/" + iAptGetCurrentDownloadFileName);
+	QFile pkgfile(KAptArchivePartialDir + "/" + iAptGetCurrentDownloadFileName);
 	iAptGetCurrentFileDownloadSize = pkgfile.size()/1024;
 
 	if( iUiDimmer && iUiDimmer->busy() ) {
@@ -820,7 +873,7 @@ QByteArray AAptInterface::readLogFile()
 {
 	QByteArray log = "";
 
-	QFile own(KLogFileName);
+	QFile own(KLogFile);
 	if( own.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		while(!own.atEnd())
@@ -844,7 +897,7 @@ void AAptInterface::logToFile( QString data, bool logtime )
 
 void AAptInterface::logToFile( QByteArray data, bool logtime )
 {
-	QFile f(KLogFileName);
+	QFile f(KLogFile);
 
 	if( f.open( QIODevice::Append | QIODevice::WriteOnly | QIODevice::Text ) )
 	{
@@ -869,7 +922,7 @@ void AAptInterface::readRepositoryInfo()
 	iRepositories.clear();
 	bool ownFound = false;
 
-	QFile own("/root/.fapman/repos.list");
+	QFile own( KOwnRepoFile );
 	if( own.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		Repository* r = 0;
@@ -892,7 +945,7 @@ void AAptInterface::readRepositoryInfo()
 			ownFound = true;
 	}
 
-	QFile names("/root/.fapman/repos.names");
+	QFile names( KOwnRepoNamesFile );
 	if( names.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		int c=0;
@@ -910,7 +963,7 @@ void AAptInterface::readRepositoryInfo()
 	if( ownFound )
 		return;
 
-	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
+	QFile ham( KHamRepoListFile );
 	if( ham.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		while(!ham.atEnd())
@@ -935,7 +988,7 @@ bool AAptInterface::writeRepositories()
 
 	iNeedListRefresh = true;
 
-	QFile own("/root/.fapman/repos.list");
+	QFile own( KOwnRepoFile );
 	if( own.open(QIODevice::WriteOnly | QIODevice::Text ) )
 	{
 		QTextStream out(&own);
@@ -944,7 +997,7 @@ bool AAptInterface::writeRepositories()
 		own.close();
 	}
 
-	QFile names("/root/.fapman/repos.names");
+	QFile names( KOwnRepoNamesFile );
 	if( names.open(QIODevice::WriteOnly | QIODevice::Text ) )
 	{
 		QTextStream out(&names);
@@ -953,7 +1006,7 @@ bool AAptInterface::writeRepositories()
 		names.close();
 	}
 
-	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
+	QFile ham( KAptSourceList );
 	if( ham.open(QIODevice::WriteOnly | QIODevice::Text ) )
 	{
 		QTextStream out(&ham);
@@ -963,7 +1016,7 @@ bool AAptInterface::writeRepositories()
 
 		return true;
 	} else {
-		qDebug() << "Failed to write repository list to sources.d!";
+		qDebug() << "Failed to write repository list!";
 	}
 
 	return false;
@@ -982,34 +1035,26 @@ bool AAptInterface::needRepoRefresh()
 		return true;
 	}
 
-	QFile own("/root/.fapman/repos.list");
-	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
+	QFile own(KAptSourceList);
 
-	if( own.exists() && ham.exists() )
+	if( !own.exists() )
 	{
-		if( own.size() != ham.size() ) {
-			qDebug() << "repo update required, debug 2";
-			return true;
-		}
+		qDebug() << "repo update required, debug 2";
+		return true;
+	}
+
+	QFileInfo a(KLastUpdateFile);
+	QDateTime aDate;
+	aDate.setTime_t(0);
+	if( a.exists() ) {
+		aDate = a.lastModified();
 	} else {
 		qDebug() << "repo update required, debug 3";
 		return true;
 	}
-
-	QFileInfo a("/root/.fapman/lastupdate");
-	QFileInfo b("/home/user/.hildon-application-manager/last-update");
-	QDateTime aDate;
-	QDateTime bDate;
-	aDate.setTime_t(0);
-	bDate.setTime_t(0);
-	if( a.exists() )
-		aDate = a.lastModified();
-	if( b.exists() )
-		bDate = b.lastModified();
 	aDate = aDate.addSecs(24*60*60); //24h
-	bDate = bDate.addSecs(24*60*60); //24h
 
-	if( aDate < QDateTime::currentDateTime() && bDate < QDateTime::currentDateTime() ) {
+	if( aDate < QDateTime::currentDateTime() ) {
 		qDebug() << "repo update required, debug 4";
 		return true;
 	}
@@ -1025,7 +1070,7 @@ void AAptInterface::startPkgListRead()
 	logToFile( QString("Start reading package lists") );
 	qDebug() << "reading package list files";
 
-	if( !iNeedListRefresh ) {
+	if( !iNeedListRefresh && !iNeedDpkgRefresh ) {
 		qDebug() << "no need to refresh package lists";
 		logToFile( QString("No need to read package lists") );
 		communicateStatusToUi(true, "Operation finished", "Package data already up to date");
@@ -1039,155 +1084,191 @@ void AAptInterface::startPkgListRead()
 	}
 
 // clear packages lists
-	QHashIterator<QString, Package*> a( iPackagesAvailable );
-	while (a.hasNext())
+	if( iNeedListRefresh && !iSkipRefreshListAndDates )
 	{
-		a.next();
-		delete a.value();
+		QHashIterator<QString, Package*> a( iPackagesAvailable );
+		while (a.hasNext())
+		{
+			a.next();
+			delete a.value();
+		}
+		iPackagesAvailable.clear();
 	}
-	iPackagesAvailable.clear();
 
-	QHashIterator<QString, Package*> i( iPackagesInstalled );
-	while (i.hasNext())
+	if( iNeedDpkgRefresh )
 	{
-		i.next();
-		delete i.value();
+		QHashIterator<QString, Package*> i( iPackagesInstalled );
+		while (i.hasNext())
+		{
+			i.next();
+			delete i.value();
+		}
+		iPackagesInstalled.clear();
 	}
-	iPackagesInstalled.clear();
 
-	iCanCancel = true;
+	// read apt database (available packages)
+	QTime time_aptread;
+	time_aptread.start();
 
-// read apt database (available packages)
 	int pkgcount_apt = 0;
-	QDir dir("/var/lib/apt/lists/");
+	QDir dir( KAptListDir );
 	QFileInfoList files = dir.entryInfoList();
 
 	quint64 totaldatasize = 0;
 	quint64 currentreaddata = 0;
 	quint64 lastupdatedata = 0;
 	quint64 updateinterval = 1000000;
-	for( int i=0; i<files.count(); i++ )
-	{
-		if( files.at(i).fileName().endsWith("_Packages"))
-			totaldatasize += files.at(i).size();
-	}
-	QFileInfo dbinfo("/var/lib/dpkg/status");
-	totaldatasize += dbinfo.size();
-	int filecount = 0;
-
-	for( int i=0; i<files.count(); i++ )
-	{
-		Repository* currentRepo = 0;
-		if( files.at(i).absoluteFilePath().endsWith("_Packages") )
+	if( iNeedListRefresh && !iSkipRefreshListAndDates ) {
+		for( int i=0; i<files.count(); i++ )
 		{
-			filecount++;
-			for(int x=0; x<iRepositories.count(); x++) {
-				if( iRepositories.at(x)->toListFileNames().contains( files.at(i).fileName() ) ) {
-					currentRepo = iRepositories.at(x);
-				}
-			}
-
-			if( iUiDimmer && currentRepo ) {
-				iUiDimmer->updateText("Reading package lists<br><font size=\"-1\">" + currentRepo->name() + "</font>");
-			}
-
-			//qDebug() << files.at(i).fileName();
-
-			QFile db( files.at(i).absoluteFilePath() );
-			if (!db.open(QIODevice::ReadOnly | QIODevice::Text)) {
-				qDebug() << "FAIL: Unable to read apt database";
-				communicateStatusToUi(false, "Error", "Unable to read package lists");
-				cleanAfterError();
-				return;
-			}
-
-			while (!db.atEnd() && !iTerminated) {
-				Package* newpkg = ReadNextPackage(db, currentreaddata);
-				//qDebug() << "read" << currentreaddata << "of" << totaldatasize;
-				if( iUiDimmer && currentreaddata >= lastupdatedata+updateinterval ) {
-					iUiDimmer->setProgress( currentreaddata*100/totaldatasize );
-					lastupdatedata = currentreaddata;
-				}
-				QApplication::processEvents();
-				pkgcount_apt++;
-				if( newpkg )
-				{
-					newpkg->addRepository( currentRepo );
-
-					Package* exists = iPackagesAvailable.value(newpkg->name(),0);
-					if( !exists ) {
-						iPackagesAvailable.insert(newpkg->name(), newpkg);
-					} else {
-						if( Package::versionCompare(newpkg->version(),exists->version()) )
-						{
-							iPackagesAvailable.remove(exists->name());
-							delete exists;
-							exists=0;
-							iPackagesAvailable.insert(newpkg->name(), newpkg);
-						} else {
-							if( newpkg->version() == exists->version() ) {
-								exists->addRepository( currentRepo );
-								if( newpkg->fullFileNames().count()>0 )
-									exists->addFullFileName( newpkg->fullFileNames().at(0) );
-								else
-									exists->addFullFileName("unknown_dir/unknown_filename");
-							}
-							delete newpkg;
-							newpkg=0;
-						}
-					}					
-				}
-			}
-			db.close();
+			if( files.at(i).fileName().endsWith("_Packages"))
+				totaldatasize += files.at(i).size();
 		}
 	}
+	if( iNeedDpkgRefresh ) {
+		QFileInfo dbinfo( KDpkgStatusFile );
+		totaldatasize += dbinfo.size();
+	}
+	int filecount = 0;
 
-	qDebug() << "Processed" << filecount << "package list files";
+	if( iNeedListRefresh && !iSkipRefreshListAndDates )
+	{
+		iCanCancel = true;
 
-	if( iTerminated ) {
-		if( iUiDimmer )
-			iUiDimmer->setProgress(-1);
-		cleanAfterError();
-		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");		
-		return;
+		for( int i=0; i<files.count(); i++ )
+		{
+			Repository* currentRepo = 0;
+			if( files.at(i).absoluteFilePath().endsWith("_Packages") )
+			{
+				filecount++;
+				for(int x=0; x<iRepositories.count(); x++) {
+					if( iRepositories.at(x)->toListFileNames().contains( files.at(i).fileName() ) ) {
+						currentRepo = iRepositories.at(x);
+					}
+				}
+
+				if( iUiDimmer && currentRepo ) {
+					iUiDimmer->updateText("Reading package lists<br><font size=\"-1\">" + currentRepo->name() + "</font>");
+				}
+
+				//qDebug() << files.at(i).fileName();
+
+				QFile db( files.at(i).absoluteFilePath() );
+				if (!db.open(QIODevice::ReadOnly | QIODevice::Text)) {
+					qDebug() << "FAIL: Unable to read apt database";
+					communicateStatusToUi(false, "Error", "Unable to read package lists");
+					cleanAfterError();
+					return;
+				}
+
+				while (!db.atEnd() && !iTerminated) {
+					Package* newpkg = ReadNextPackage(db, currentreaddata);
+					//qDebug() << "read" << currentreaddata << "of" << totaldatasize;
+					if( iUiDimmer && currentreaddata >= lastupdatedata+updateinterval ) {
+						iUiDimmer->setProgress( currentreaddata*100/totaldatasize );
+						lastupdatedata = currentreaddata;
+						QApplication::processEvents();
+					}
+					pkgcount_apt++;
+					if( newpkg )
+					{
+						newpkg->addRepository( currentRepo );
+
+						Package* exists = iPackagesAvailable.value(newpkg->name(),0);
+						if( !exists ) {
+							iPackagesAvailable.insert(newpkg->name(), newpkg);
+						} else {
+							if( Package::versionCompare(newpkg->version(),exists->version()) )
+							{
+								iPackagesAvailable.remove(exists->name());
+								delete exists;
+								exists=0;
+								iPackagesAvailable.insert(newpkg->name(), newpkg);
+							} else {
+								if( newpkg->version() == exists->version() ) {
+									exists->addRepository( currentRepo );
+									if( newpkg->fullFileNames().count()>0 )
+										exists->addFullFileName( newpkg->fullFileNames().at(0) );
+									else
+										exists->addFullFileName("unknown_dir/unknown_filename");
+								}
+								delete newpkg;
+								newpkg=0;
+							}
+						}
+					}
+				}
+				db.close();
+			}
+		}
+
+		qDebug() << "apt database read took" << time_aptread.elapsed() << "ms";
+		qDebug() << "Processed" << filecount << "package list files";
+
+		if( iTerminated ) {
+			if( iUiDimmer )
+				iUiDimmer->setProgress(-1);
+			cleanAfterError();
+			communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
+			return;
+		}
+
+		iNeedListRefresh = false;
+		iNeedDateRefresh = true;
 	}
 
 // read dpkg database (installed packages)
-	int pkgcount_dpkg = 0;
-	QFile db("/var/lib/dpkg/status");
-	if (!db.open(QIODevice::ReadOnly | QIODevice::Text)) {
-		qDebug() << "FAIL: Unable to read dpkg database";
-		communicateStatusToUi(false, "Error", "Unable to read package database");
-		cleanAfterError();
-		return;
-	}
+	if( iNeedDpkgRefresh )
+	{
+		iCanCancel = true;
+		QTime time_dpkgread;
+		time_dpkgread.start();
 
-	if( iUiDimmer ) {
-		iUiDimmer->updateText("Reading package lists<br><font size=\"-1\">dpkg database</font>");
-	}
-
-	while (!db.atEnd() && !iTerminated) {
-		Package* newpkg = ReadNextPackage(db, currentreaddata);
-		if( iUiDimmer && currentreaddata >= lastupdatedata+updateinterval ) {
-			iUiDimmer->setProgress( currentreaddata*100/totaldatasize );
-			lastupdatedata = currentreaddata;
+		int pkgcount_dpkg = 0;
+		QFile db( KDpkgStatusFile );
+		if (!db.open(QIODevice::ReadOnly | QIODevice::Text)) {
+			qDebug() << "FAIL: Unable to read dpkg database";
+			communicateStatusToUi(false, "Error", "Unable to read package database");
+			cleanAfterError();
+			return;
 		}
-		QApplication::processEvents();
-		pkgcount_dpkg++;
-		if( newpkg ) {
-			if( newpkg->isInstalled() ) {
-				iPackagesInstalled.insert(newpkg->name(), newpkg);
-			} else {
-				delete newpkg;
-				newpkg=0;
+
+		if( iUiDimmer ) {
+			iUiDimmer->updateText("Reading package lists<br><font size=\"-1\">dpkg database</font>");
+		}
+
+		while (!db.atEnd() && !iTerminated) {
+			Package* newpkg = ReadNextPackage(db, currentreaddata);
+			if( iUiDimmer && currentreaddata >= lastupdatedata+updateinterval ) {
+				iUiDimmer->setProgress( currentreaddata*100/totaldatasize );
+				lastupdatedata = currentreaddata;
+				QApplication::processEvents();
+			}
+			pkgcount_dpkg++;
+			if( newpkg ) {
+				if( newpkg->isInstalled() ) {
+					iPackagesInstalled.insert(newpkg->name(), newpkg);
+				} else {
+					delete newpkg;
+					newpkg=0;
+				}
 			}
 		}
-	}
-	db.close();
+		db.close();
 
-	qDebug() << "Processed" << pkgcount_apt << "(apt) and" << pkgcount_dpkg << "(dpkg) package entries";
-	qDebug() << "In DB:" << iPackagesAvailable.count() << "packages available,"
-			<< iPackagesInstalled.count() << "installed";
+		qDebug() << "dpkg database read took" << time_dpkgread.elapsed() << "ms";
+
+		qDebug() << "Processed" << pkgcount_apt << "(apt) and" << pkgcount_dpkg << "(dpkg) package entries";
+		qDebug() << "In DB:" << iPackagesAvailable.count() << "packages available,"
+				<< iPackagesInstalled.count() << "installed";
+
+		if( iTerminated ) {
+			cleanAfterError();
+			communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
+			return;
+		}
+		iNeedDpkgRefresh = false;
+	}
 
 	logToFile( QString("Finished reading package lists") );
 	if( iUiDimmer ) {
@@ -1196,18 +1277,10 @@ void AAptInterface::startPkgListRead()
 		iUiDimmer->setProgress(-1);
 	}
 
-	if( iTerminated ) {
-		cleanAfterError();
-		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
-		return;
-	}
-
 	readBlacklist();
 
 	communicateStatusToUi(true, "Operation finished", "Package data read");
 	iCanCancel = false;
-	iNeedListRefresh = false;
-	iNeedDateRefresh = true;
 	runNext();
 }
 
@@ -1227,7 +1300,7 @@ Package* AAptInterface::ReadNextPackage(QFile& f, quint64& currentreaddata)
 	}
 
 	if( pkg->name() != "" && pkg->isInstalled() ) {
-		QFileInfo f( "/var/lib/dpkg/info/" + pkg->name() + ".list" );
+		QFileInfo f( KDpkgInfoDir + "/" + pkg->name() + ".list" );
 		if( f.exists() )
 			pkg->setDate( f.lastModified() );
 	}
@@ -1282,7 +1355,7 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 	}
 	else if( line.startsWith("Maemo-Display-Name: ") )
 	{
-		pkg->setMaemoDisplayName( QString::fromUtf8(line.mid(20)) );
+		pkg->setMaemoDisplayName( line.mid(20) );
 	}
 	else if( line.startsWith("Depends: ") )
 	{
@@ -1296,7 +1369,7 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 	if( iMultiLine == MultiLineDesc ) {
 		if( (line.startsWith(' ') || line.startsWith('\t')) && line.trimmed()!="" ) {
 			if( line.trimmed()!="." )
-				pkg->appendDescLong( QString::fromUtf8(line.trimmed()) + "\n" );
+				pkg->appendDescLong( line.trimmed() + "\n" );
 			else
 				pkg->appendDescLong( "\n" );
 		} else {
@@ -1310,10 +1383,17 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 			iMultiLine = MultiLineNone;
 		}
 	}
+	else if( iMultiLine == MultiLineUpgradeDesc ) {
+		if( (line.startsWith(' ') || line.startsWith('\t')) && line.trimmed()!="" ) {
+			pkg->appendUpgradeDescription( line.trimmed() + "\n" );
+		} else {
+			iMultiLine = MultiLineNone;
+		}
+	}
 
-	if( line.startsWith("Description: ") )
+	if( line.startsWith("Description:") )
 	{
-		pkg->setDescShort( QString::fromUtf8(line.mid(13).trimmed()) );
+		pkg->setDescShort( line.mid(12).trimmed() );
 		iMultiLine = MultiLineDesc;
 	}
 	else if( line.startsWith("Maemo-Icon-26:") )
@@ -1323,6 +1403,11 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 		}
 		iMultiLine = MultiLineIcon;
 	}
+	else if( line.startsWith("Maemo-Upgrade-Description:") )
+	{
+		pkg->appendUpgradeDescription( line.mid(26).trimmed() + "\n" );
+		iMultiLine = MultiLineUpgradeDesc;
+	}
 
 	return false;
 }
@@ -1344,9 +1429,22 @@ void AAptInterface::writeBlacklist()
 		}
 	}
 
+	QHashIterator<QString, Package*> j( iPackagesInstalled );
+	while (j.hasNext())
+	{
+		j.next();
+
+		if( j.value()->blacklisted() == BlacklistSelect::BlacklistAll ) {
+			iBlacklist << j.value()->name();
+		}
+		else if( j.value()->blacklisted() == BlacklistSelect::BlacklistThis ) {
+			iBlacklist << (j.value()->name() + " " + j.value()->version());
+		}
+	}
+
 	iBlacklist.removeDuplicates();
 
-	QFile f("/root/.fapman/black.list");
+	QFile f( KBlacklistFile );
 	if( f.open(QIODevice::WriteOnly | QIODevice::Text ) )
 	{
 		QTextStream out(&f);
@@ -1360,7 +1458,7 @@ void AAptInterface::writeBlacklist()
 
 void AAptInterface::readBlacklist()
 {
-	QFile f("/root/.fapman/black.list");
+	QFile f( KBlacklistFile );
 	if( f.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		while( !f.atEnd() ) {
@@ -1378,16 +1476,18 @@ void AAptInterface::readBlacklist()
 	for( int i=0; i<iBlacklist.count(); i++ )
 	{
 		QStringList parts = iBlacklist.at(i).split(' ');
+		Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
+		Package* pkg2 = iPackagesInstalled.value(parts.at(0).trimmed(), 0);
 		if( parts.count()==1 ) {
-			Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
-			if( pkg ) {
+			if( pkg && !pkg2 )
 				pkg->setBlacklisted(BlacklistSelect::BlacklistAll);
-			}
+			if( pkg2 )
+				pkg2->setBlacklisted(BlacklistSelect::BlacklistAll);
 		} else if( parts.count()==2 ) {
-			Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
-			if( pkg && pkg->version()==parts.at(1) ) {
+			if( pkg && pkg->version()==parts.at(1) )
 				pkg->setBlacklisted(BlacklistSelect::BlacklistThis);
-			}
+			if( pkg2 && pkg2->version()==parts.at(1) )
+				pkg2->setBlacklisted(BlacklistSelect::BlacklistThis);
 		} else {
 			qDebug() << "Warning: invalid blacklist entry:" << iBlacklist.at(i);
 		}
@@ -1432,14 +1532,13 @@ void AAptInterface::startFetchDates()
 	logToFile( QString("Start fetching package dates") );
 	qDebug() << "start fetching package dates";
 
-	if( !iNeedDateRefresh ) {
+	if( !iNeedDateRefresh || iSkipRefreshListAndDates ) {
 		qDebug() << "no need to fetch dates";
 		logToFile( QString("No need to fetch dates") );
 		communicateStatusToUi(true, "Operation finished", "Date information already up to date");
 		runNext();
 		return;
 	}
-	iNeedListRefresh = true;
 
 	if( iUiDimmer ) {
 		iUiDimmer->updateText("Reading date cache");
@@ -1487,7 +1586,9 @@ void AAptInterface::startFetchDates()
 	iDateRequestsReceived = 0;
 	iNetworkError = QNetworkReply::NoError;
 
-	qDebug() << "package dates to fetch:" << fetchable.count();
+	QString dbgcount = QString("need to fetch date for %1 packages").arg(fetchable.count());
+	qDebug() << dbgcount;
+	logToFile(dbgcount);
 
 	QHashIterator<QString, Package*> fe( fetchable );
 	while (fe.hasNext() && !iTerminated)
@@ -1538,6 +1639,7 @@ void AAptInterface::startFetchDates()
 					iNetworkError != QNetworkReply::QNetworkReply::ContentAccessDenied && iNetworkError != QNetworkReply::QNetworkReply::ContentNotFoundError )
 				{
 					qDebug() << "fatal network error, aborting fetch";
+					logToFile(QString("Fatal network error, date fetch aborted"));
 					break;
 				} else {
 					nam->head(r);
@@ -1581,8 +1683,10 @@ void AAptInterface::startFetchDates()
 		QApplication::processEvents();
 	}
 
-	qDebug() << "sent" << iDateRequestsSent << "requests, received" << iDateRequestsReceived << "replies with"
-			<< iDateRequestErrors << "errors";
+	QString dbgstr = QString("sent %1 requests, received %2 replies with %3 errors").arg(iDateRequestsSent).arg(iDateRequestsReceived).arg(iDateRequestErrors);
+
+	qDebug() << dbgstr;
+	logToFile(dbgstr);
 
 	logToFile( QString("Finished fetching package dates") );
 
@@ -1600,7 +1704,6 @@ void AAptInterface::startFetchDates()
 
 	communicateStatusToUi(true, "Operation finished", "Package dates fetched");
 	iNeedDateRefresh = false;
-	iNeedListRefresh = false;
 	runNext();
 }
 
@@ -1628,7 +1731,9 @@ void AAptInterface::dateFetchNetworkReply(QNetworkReply* reply)
 	}
 
 	if( reply->error() != QNetworkReply::NoError ) {
-		qDebug() << reply->error() << reply->errorString();
+		QString dbg = reply->url().fragment() + QString(": error %1: ").arg(reply->error()) + reply->errorString();
+		qDebug() << dbg;
+		logToFile( dbg );
 		iDateRequestErrors++;
 	}
 	reply->deleteLater();
@@ -1638,7 +1743,7 @@ void AAptInterface::writeDateCache()
 {
 	qDebug() << "writing date cache";
 
-	QFile f("/root/.fapman/dates.cache");
+	QFile f(KDateCacheFile);
 	if( f.open(QIODevice::WriteOnly | QIODevice::Text ) )
 	{
 		QTextStream out(&f);
@@ -1652,6 +1757,7 @@ void AAptInterface::writeDateCache()
 		f.close();
 	} else {
 		qDebug() << "Warning: failed to write date cache";
+		logToFile(QString("Failed to write date cache"));
 	}
 }
 
@@ -1659,7 +1765,7 @@ void AAptInterface::readDateCache()
 {
 	qDebug() << "reading date cache";
 
-	QFile f("/root/.fapman/dates.cache");
+	QFile f(KDateCacheFile);
 	if( f.open(QIODevice::ReadOnly | QIODevice::Text ) )
 	{
 		while( !f.atEnd() ) {
@@ -1674,10 +1780,12 @@ void AAptInterface::readDateCache()
 						pkg->setDate( dt );
 					} else {
 						qDebug() << "Warning: Invalid date in date cache";
+						logToFile(QString("Invalid date in date cache"));
 					}
 				}
 			} else {
 				qDebug() << "Warning: error in date cache:" << line;
+				logToFile(QString("Error in date cache"));
 			}
 		}
 		f.close();
@@ -1688,7 +1796,13 @@ void AAptInterface::readDateCache()
 
 bool AAptInterface::dateCacheExists()
 {
-	QFileInfo f("/root/.fapman/dates.cache");
-
+	QFileInfo f(KDateCacheFile);
 	return f.exists();
 }
+
+bool AAptInterface::loadInstallFiles(QStringList files_)
+{
+	qDebug() << files_;
+
+	return false;
+}
