@@ -24,8 +24,24 @@
 #include <QtNetwork>
 #include "blacklistselect.h"
 
-const QString KLogFileDir = "/root/.fapman";
-const QString KLogFileName = "/root/.fapman/lastlog.txt";
+
+const QString KFapmanDir = "/root/.fapman";
+const QString KLogFile = "/root/.fapman/lastlog.txt";
+const QString KBlacklistFile = "/root/.fapman/black.list";
+const QString KOwnRepoFile = "/root/.fapman/repos.list";
+const QString KOwnRepoNamesFile = "/root/.fapman/repos.names";
+
+const QString KHamRepoListFile = "/etc/apt/sources.list.d/hildon-application-manager.list";
+
+const QString KLastUpdateFile = "/opt/fapman-cache/lastupdate";
+const QString KDateCacheFile = "/opt/fapman-cache/dates.cache";
+const QString KAptSourceList = "/opt/fapman-cache/sources.list";
+const QString KAptListDir = "/opt/fapman-cache/lists";
+const QString KAptArchivePartialDir = "/var/cache/apt/archives/partial";
+
+const QString KDpkgStatusFile = "/var/lib/dpkg/status";
+const QString KDpkgInfoDir = "/var/lib/dpkg/info";
+
 
 class MainWindow;
 class FileReadThread;
@@ -43,7 +59,7 @@ class AAptInterface : public QObject
 public:
 	enum interfaceMode { ModeNone, ModeReadPackages, ModeFetchDates, ModeAptGetUpdate,
 						 ModeAptGetSimulate, ModeAptGetInstall, ModeAptGetClean };
-	enum multiLine { MultiLineNone, MultiLineDesc, MultiLineIcon };
+	enum multiLine { MultiLineNone, MultiLineDesc, MultiLineIcon, MultiLineUpgradeDesc };
 
 	AAptInterface(QObject* parent);
 	~AAptInterface();
@@ -74,8 +90,11 @@ public:
 
 	bool dateCacheExists();
 
-	void setNeedListRefresh(bool refresh) { iNeedListRefresh=refresh; }
-	//bool needListRefresh() { return iNeedListRefresh; }
+	void setNeedRefresh(int repos, int lists, int dpkg, int dates);
+	void setSkipListAndDates() { iSkipRefreshListAndDates=true; }
+	bool needListOrDateRefresh();
+
+	bool loadInstallFiles(QStringList files_);
 
 private:
 	void runNext();
@@ -144,7 +163,9 @@ private:
 	bool iErrorDone;
 	bool iNeedRepoRefresh;
 	bool iNeedListRefresh;
+	bool iNeedDpkgRefresh;
 	bool iNeedDateRefresh;
+	bool iSkipRefreshListAndDates;
 
 	QProcess* iProcAptGetUpdate;
 	QProcess* iProcAptGetSimulate;
