@@ -40,6 +40,9 @@ int main(int argc, char *argv[])
 	qInstallMsgHandler( myMessageOutput );
     QApplication a(argc, argv);
 
+	a.setApplicationName("faster_application_manager");
+	QDir::setCurrent("/root/.fapman/");
+
 	EnableDebugOutput = false;
 	if( a.arguments().contains("-d") ) {
 		EnableDebugOutput = true;
