@@ -24,7 +24,7 @@
 #include "apt-src/debversion.h"
 #include "blacklistselect.h"
 
-Package::Package(QString name_, AAptInterface *apt_)
+Package::Package(QByteArray name_, AAptInterface *apt_)
 {
 	iAptInterface = apt_;
 	iName = name_;
@@ -46,13 +46,13 @@ Package::~Package()
 
 QString Package::displayName()
 {
-	QString name = iName;
+	QString pkgname = name();
 	if( iMaemoDisplayName!="" )
-		name = iMaemoDisplayName;
-	QString n( name.at(0) );
+		pkgname = maemoDisplayName();
+	QString n( pkgname.at(0) );
 	n = n.toUpper();
-	name.replace(0,1,n);
-	return name;
+	pkgname.replace(0,1,n);
+	return pkgname;
 }
 
 QString Package::fileName()
@@ -168,25 +168,27 @@ bool Package::versionCompare(QString isNewer, QString compare)
 	return false;
 }
 
-void Package::setDepends(QString depends_)
+void Package::setDepends(QByteArray depends_)
 {
 	iDepends = depends_.split(',');
 }
 
-void Package::setConflicts(QString conflicts_)
+void Package::setConflicts(QByteArray conflicts_)
 {
 	iConflicts = conflicts_.split(',');
 }
 
 QStringList Package::depends()
 {
-	qDebug() << iName;
-	qDebug() << iDepends;
+	//qDebug() << iName;
+	//qDebug() << iDepends;
+
 	QStringList list;
 
-	for(int i=0; i<iConflicts.count(); i++)
+	for(int i=0; i<iDepends.count(); i++)
 	{
-		QStringList parts = iConflicts.at(i).split('|');
+		/*
+		QList<QByteArray> parts = iConflicts.at(i).split('|');
 		for(int j=0; j<parts.count(); j++)
 		{
 			qDebug() << parts.at(j);
@@ -198,6 +200,8 @@ QStringList Package::depends()
 				s = parts.at(j).left(pos).trimmed();
 			list << s;
 		}
+		*/
+		list << iDepends.at(i).trimmed();
 	}
 
 	return list;
@@ -207,6 +211,10 @@ QStringList Package::conflicts()
 {
 	QStringList list;
 
+	for(int i=0; i<iConflicts.count(); i++)
+	{
+		list << iConflicts.at(i).trimmed();
+	}
 
 	return list;
 }
