@@ -17,8 +17,10 @@
 	(C) Heikki Holstila 2010
 */
 
+#include <QtNetwork>
 #include <QDBusConnection>
 #include <QDBusInterface>
+
 #include "packageselector.h"
 #include "ui_packageselector.h"
 #include "package.h"
@@ -66,7 +68,8 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 	}
 
 	Package* upg_pkg = iPkg->availablePackage();
-	if( !pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="") )
+	//if( (!pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="")) && getMaemoOrgUrl(iPkg)=="" )
+	if( (!pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="")) )
 		ui->radioTabChanges->hide();
 
 	if( pkg->isUpgradeable() )
@@ -125,12 +128,16 @@ Package::operation PackageSelector::selectedOperation()
 
 void PackageSelector::on_pushButton_blacklist_clicked()
 {
-	BlacklistSelect s(iPkg, this);
-	BlacklistSelect::blackList old = iPkg->blacklisted();
+	Package* pkg = iPkg;
+	/*if( iPkg->isUpgradeable() && iPkg->availablePackage() )
+		pkg = iPkg->availablePackage();*/
+
+	BlacklistSelect s(pkg, this);
+	BlacklistSelect::blackList old = pkg->blacklisted();
 	if( s.exec() ) {
 		updateInfo();
-		if( old != iPkg->blacklisted() ) {
-			iAptInterface->removeFromBlacklist(iPkg, old);
+		if( old != pkg->blacklisted() ) {
+			iAptInterface->removeFromBlacklist(pkg, old);
 			iAptInterface->writeBlacklist();
 		}
 	}
@@ -216,25 +223,6 @@ void PackageSelector::on_radioTabBasic_clicked()
 		header2 += "<br>";
 	}
 
-	ui->radioInstall->setEnabled(true);
-	ui->radioRemove->setEnabled(true);
-	ui->radioInstall->show();
-	ui->radioRemove->show();
-
-	if( iPkg->isInstalled() && !iPkg->isUpgradeable() )
-	{
-		ui->radioInstall->setEnabled(false);
-		ui->radioInstall->hide();
-	} else if( !iPkg->isInstalled() ) {
-		ui->radioRemove->setEnabled(false);
-		ui->radioRemove->hide();
-	}
-
-	if( iPkg->isBlacklisted() ) {
-		ui->radioInstall->setEnabled(false);
-		ui->radioRemove->setEnabled(false);
-	}
-
 	ui->label_header2->setText(header2);
 
 
@@ -328,25 +316,65 @@ void PackageSelector::on_radioTabBasic_clicked()
 	}
 
 	ui->label_text->setText( longtext );
+
+	ui->radioInstall->setEnabled(true);
+	ui->radioRemove->setEnabled(true);
+	ui->radioInstall->show();
+	ui->radioRemove->show();
+
+	if( iPkg->isInstalled() && !iPkg->isUpgradeable() )
+	{
+		ui->radioInstall->setEnabled(false);
+		ui->radioInstall->hide();
+	} else if( !iPkg->isInstalled() ) {
+		ui->radioRemove->setEnabled(false);
+		ui->radioRemove->hide();
+	}
+
+	if( iPkg->isBlacklisted() ) {
+		ui->radioInstall->setEnabled(false);
+		ui->radioRemove->setEnabled(false);
+	}
 }
 
 void PackageSelector::on_radioTabChanges_clicked()
 {
+	/*
 	ui->label_header2->show();
-	ui->label_header2->setText("Upgrade description:");
 	ui->label_text->setText("");
+	ui->label_header2->setText("Fetching changelog...");
+	QApplication::processEvents();
 
+	QString changelog;
+	if( getMaemoOrgUrl(iPkg)!="" )
+		 changelog = queryChangelog();
+	*/
 	Package* upg_pkg = iPkg->availablePackage();
 	if( !upg_pkg )
 		upg_pkg = iPkg;
 
-	if( upg_pkg ) {
-		QString udesc = "<br><font size=\"-1\">";
-		udesc += upg_pkg->upgradeDescription();
-		udesc += "</font>";
-		udesc.replace('\n',"<br>");
-		ui->label_text->setText(udesc);
+	QString text;
+	if( iPkg->isUpgradeable() && upg_pkg && upg_pkg->upgradeDescription()!="" )
+	{
+		text += "Upgrade description:";
+		text += "<font size=\"-1\"><br><br>";
+		text += upg_pkg->upgradeDescription();
+		text += "</font><br><br>";
+		text.replace('\n',"<br>");
 	}
+
+	/*
+	text += "Debian changelog:<font size=\"-1\"><br><br>";
+	if( changelog != "" )
+		text += changelog;
+	else
+		text += "Not available";
+	text += "</font>";
+	*/
+
+	ui->label_header2->hide();
+	ui->label_header2->setText("");
+	ui->label_text->setText(text);
 }
 
 void PackageSelector::on_radioTabDepends_clicked()
@@ -370,3 +398,33 @@ void PackageSelector::on_radioTabDepends_clicked()
 
 	ui->label_text->setText(text);
 }
+
+QString PackageSelector::queryChangelog()
+{
+	QUrl url( getMaemoOrgUrl(iPkg) );
+	QNetworkAccessManager nam(this);
+	QNetworkRequest req(url);
+
+	QNetworkReply* reply = nam.get(req);
+
+	qDebug() << "fetching changelog";
+
+	while(!reply->isFinished()) {
+		QApplication::processEvents(QEventLoop::WaitForMoreEvents);
+		//if(  )
+		//reply->abort();
+	}
+
+	if( reply->error() != QNetworkReply::NoError ) {
+		qDebug() << "changelog fetch error:" << reply->errorString();
+		delete reply;
+		return "";
+	}
+
+	QByteArray data = reply->readAll();
+	delete reply;
+
+
+	qDebug() << "success";
+	return "";
+}
