@@ -50,20 +50,23 @@ public:
 
 	Package::operation selectedOperation();
 
-
 protected:
     void changeEvent(QEvent *e);
 
 private:
 	QString getMaemoOrgUrl(Package* pkg);
-	void updateHeader2();
+	void updateInfo();
+	QString queryChangelog();
 
     Ui::PackageSelector *ui;
 	Package* iPkg;
 	AAptInterface* iAptInterface;
 
 private slots:
-	void on_pushButton_website_clicked();
+	void on_radioTabDepends_clicked();
+ void on_radioTabChanges_clicked();
+ void on_radioTabBasic_clicked();
+ void on_pushButton_website_clicked();
 	void on_pushButton_blacklist_clicked();
 };
 
